//#include <Ethernet.h>

// Adafruit_NeoMatrix example for single NeoPixel Shield.
// By Marc MERLIN <marc_soft@merlins.org>
// Contains code (c) Adafruit, license BSD

#include <Adafruit_GFX.h>
#include <Adafruit_NeoMatrix.h>
#include <Adafruit_NeoPixel.h>
#include "TimerOne.h"

// Choose your prefered pixmap
//#include "heart24.h"
//#include "yellowsmiley24.h"
//#include "bluesmiley24.h"
//#include "smileytongue24.h"
#ifndef PSTR
 #define PSTR // Make Arduino Due happy
#endif

#define PIN 6

// ESP8266 has an I2S neopixel library which can only use pin RX
// so it's recommended to use the same pin with Neopixel to avoid
// rewiring when changing libs
#ifdef ESP8266
#define PIN RX
#endif

// Max is 255, 32 is a conservative value to not overload
// a USB power supply (500mA) for 12x12 pixels.
#define BRIGHTNESS 120

// Define matrix width and height.
#define mw 11
#define mh 10

// MATRIX DECLARATION:
// Parameter 1 = width of EACH NEOPIXEL MATRIX (not total display)
// Parameter 2 = height of each matrix
// Parameter 3 = number of matrices arranged horizontally
// Parameter 4 = number of matrices arranged vertically
// Parameter 5 = pin number (most are valid)
// Parameter 6 = matrix layout flags, add together as needed:
//   NEO_MATRIX_TOP, NEO_MATRIX_BOTTOM, NEO_MATRIX_LEFT, NEO_MATRIX_RIGHT:
//     Position of the FIRST LED in the FIRST MATRIX; pick two, e.g.
//     NEO_MATRIX_TOP + NEO_MATRIX_LEFT for the top-left corner.
//   NEO_MATRIX_ROWS, NEO_MATRIX_COLUMNS: LEDs WITHIN EACH MATRIX are
//     arranged in horizontal rows or in vertical columns, respectively;
//     pick one or the other.
//   NEO_MATRIX_PROGRESSIVE, NEO_MATRIX_ZIGZAG: all rows/columns WITHIN
//     EACH MATRIX proceed in the same order, or alternate lines reverse
//     direction; pick one.
//   NEO_TILE_TOP, NEO_TILE_BOTTOM, NEO_TILE_LEFT, NEO_TILE_RIGHT:
//     Position of the FIRST MATRIX (tile) in the OVERALL DISPLAY; pick
//     two, e.g. NEO_TILE_TOP + NEO_TILE_LEFT for the top-left corner.
//   NEO_TILE_ROWS, NEO_TILE_COLUMNS: the matrices in the OVERALL DISPLAY
//     are arranged in horizontal rows or in vertical columns, respectively;
//     pick one or the other.
//   NEO_TILE_PROGRESSIVE, NEO_TILE_ZIGZAG: the ROWS/COLUMS OF MATRICES
//     (tiles) in the OVERALL DISPLAY proceed in the same order for every
//     line, or alternate lines reverse direction; pick one.  When using
//     zig-zag order, the orientation of the matrices in alternate rows
//     will be rotated 180 degrees (this is normal -- simplifies wiring).
//   See example below for these values in action.
// Parameter 7 = pixel type flags, add together as needed:
//   NEO_RGB     Pixels are wired for RGB bitstream (v1 pixels)
//   NEO_GRB     Pixels are wired for GRB bitstream (v2 pixels)
//   NEO_KHZ400  400 KHz bitstream (e.g. FLORA v1 pixels)
//   NEO_KHZ800  800 KHz bitstream (e.g. High Density LED strip)

Adafruit_NeoMatrix *matrix = new Adafruit_NeoMatrix(mw, mh, PIN,
  NEO_MATRIX_TOP     + NEO_MATRIX_LEFT +
  NEO_MATRIX_ROWS + NEO_MATRIX_ZIGZAG,
  NEO_GRB            + NEO_KHZ800);

// This could also be defined as matrix->color(255,0,0) but those defines
// are meant to work for adafruit_gfx backends that are lacking color()
#define LED_BLACK		0

#define LED_RED_VERYLOW 	(3 <<  11)
#define LED_RED_LOW 		(7 <<  11)
#define LED_RED_MEDIUM 		(15 << 11)
#define LED_RED_HIGH 		(31 << 11)

#define LED_GREEN_VERYLOW	(1 <<  5)   
#define LED_GREEN_LOW 		(15 << 5)  
#define LED_GREEN_MEDIUM 	(31 << 5)  
#define LED_GREEN_HIGH 		(63 << 5)  

#define LED_BLUE_VERYLOW	3
#define LED_BLUE_LOW 		7
#define LED_BLUE_MEDIUM 	15
#define LED_BLUE_HIGH 		31

#define LED_ORANGE_VERYLOW	(LED_RED_VERYLOW + LED_GREEN_VERYLOW)
#define LED_ORANGE_LOW		(LED_RED_LOW     + LED_GREEN_LOW)
#define LED_ORANGE_MEDIUM	(LED_RED_MEDIUM  + LED_GREEN_MEDIUM)
#define LED_ORANGE_HIGH		(LED_RED_HIGH    + LED_GREEN_HIGH)

#define LED_PURPLE_VERYLOW	(LED_RED_VERYLOW + LED_BLUE_VERYLOW)
#define LED_PURPLE_LOW		(LED_RED_LOW     + LED_BLUE_LOW)
#define LED_PURPLE_MEDIUM	(LED_RED_MEDIUM  + LED_BLUE_MEDIUM)
#define LED_PURPLE_HIGH		(LED_RED_HIGH    + LED_BLUE_HIGH)

#define LED_CYAN_VERYLOW	(LED_GREEN_VERYLOW + LED_BLUE_VERYLOW)
#define LED_CYAN_LOW		(LED_GREEN_LOW     + LED_BLUE_LOW)
#define LED_CYAN_MEDIUM		(LED_GREEN_MEDIUM  + LED_BLUE_MEDIUM)
#define LED_CYAN_HIGH		(LED_GREEN_HIGH    + LED_BLUE_HIGH)

#define LED_WHITE_VERYLOW	(LED_RED_VERYLOW + LED_GREEN_VERYLOW + LED_BLUE_VERYLOW)
#define LED_WHITE_LOW		(LED_RED_LOW     + LED_GREEN_LOW     + LED_BLUE_LOW)
#define LED_WHITE_MEDIUM	(LED_RED_MEDIUM  + LED_GREEN_MEDIUM  + LED_BLUE_MEDIUM)
#define LED_WHITE_HIGH		(LED_RED_HIGH    + LED_GREEN_HIGH    + LED_BLUE_HIGH)

// This Enum contains all possible words (Sorry this works anyhow only for German Words ;)
enum WordId {
  eEs,
  eIst,
  eFuenf,
  eZehn,
  eZwanzig,
  eDrei,
  eViertel,
  eNach,
  eVor,
  eJa,
  eHalb,
  eBL,
  eZwoelfUhr,
  eZweiUhr,
  eEinUhr,
  eEinsUhr,
  eSiebenUhr,
  eDreiUhr,
  eJL,
  eFuenfUhr,
  eElfUhr,
  eNeunUhr,
  eVierUhr,
  eAchtUhr,
  eZehnUhr,
  eLL,
  eSechsUhr,
  eUhr
};

enum ColorId {
  eOff,
  eGreenLow,
  eGreenHigh
};

int myColor = LED_WHITE_HIGH + 0L; // This is the main color for display
int myHalfDays = 0;           // This is the half Day Count
byte myHours = 0;              // This is the Hour Count
byte myMinutes = 0;            // This is the Minute Count
byte mySeconds = 0;            // This is the second count
byte incomingByte = 0;         // for incoming serial data
byte incomingByteCount = 0;    // count for incoming serial data
byte myTemp = 0;              // variable for temp data
byte myMinutesLastDisplayed = 0;           // This is the Minute Count which was last Displayed
byte xPos = 0;
byte yPos = 0;


bool DebugIsActive = false;   // Shows more Debug Info
bool ClockIsActive = false;   // if false, the time will not be updated
bool CommandComplete = false; // if false, the receiving will be continued

//bool pixelArray[11][10]; // Array to represent pixels of Screen
/*
void display_circles() {
    matrix->clear();
    matrix->drawCircle(mw/2,mh/2, 2, LED_RED_MEDIUM);
    matrix->drawCircle(mw/2-1-min(mw,mh)/8, mh/2-1-min(mw,mh)/8, min(mw,mh)/4, LED_BLUE_HIGH);
    matrix->drawCircle(mw/2+1+min(mw,mh)/8, mh/2+1+min(mw,mh)/8, min(mw,mh)/4-1, LED_ORANGE_MEDIUM);
    matrix->drawCircle(1,mh-2, 1, LED_GREEN_LOW);
    matrix->drawCircle(mw-2,1, 1, LED_GREEN_HIGH);
    if (min(mw,mh)>12) matrix->drawCircle(mw/2-1, mh/2-1, min(mh/2-1,mw/2-1), LED_CYAN_HIGH);
    matrix->show();
}
*/

void sendInfoBySerial()
{
  Serial.println(F("*************************************************"));
  Serial.println(F("*    Customized Wordclock by Stefan XP (2019)   *"));
  Serial.println(F("*            Version 3 2019-01-03               *"));
  Serial.println(F("*************************************************"));
  Serial.println(F("* Available Commands:                           *"));
  Serial.println(F("* t .. Set the time by ""hh:mm:ss""                 *"));                
  Serial.println(F("* d .. Enables / Disables DebugMode             *"));                
  Serial.println(F("* r .. Send the actual Time by Serial           *"));
  Serial.println(F("* ? .. Send Help & Info by Serial               *"));
  Serial.println(F("* k .. Display Circles            *"));
  Serial.println(F("* h .. halt/continue the clock updating         *"));
  Serial.println(F("* c .. Set up the Clock Color by ""RRRGGGBBB""      *"));
  Serial.println(F("* l .. Show each Line of clock in clock color   *"));
  Serial.println(F("* m .. Show tHe mAtriX                          *"));
  Serial.println(F("* m .. Show the Rainbow                         *"));
  Serial.println(F("* x .. Clear Clock face                         *"));
  Serial.println(F("* f .. fill screen with clock color             *"));
  Serial.println(F("* p .. print point ""xxyy"" (from 0)                *"));
  Serial.println(F("* b .. set brightness (from 0 - 255)            *"));
  Serial.println(F("* u .. shows the time once                      *"));
  Serial.println(F("* n .. shows the Rainbow                        *"));
  Serial.println(F("*************************************************"));
}

void sendActualTimeBySerial()
{
  // Write via Serial
  Serial.print(F("It's: "));
  if(myHours < 10) Serial.write("0");
  Serial.print(myHours, DEC);
  Serial.write(":");
  if(myMinutes < 10) Serial.write("0");
  Serial.print(myMinutes, DEC);
  Serial.write(":");
  if(mySeconds < 10) Serial.write("0");
  Serial.print(mySeconds, DEC);
  Serial.write(" Running since [d]: ");
  Serial.print(myHalfDays/2, DEC);
  Serial.write("\n\r");
}

void ShowEachLine(int myDelay)
{
  byte iCount;
  Serial.println(" Showing each Line ");
  for(iCount = 0 ; iCount <=10; iCount++)
  {
    //Serial.println(iCount);
    matrix->fillRect(0,iCount, 11,1, myColor);
    matrix->show();
    delay(myDelay);
    matrix->fillRect(0,iCount, 11,1, 0);
    //matrix->show();
    //delay(myDelay);
  }
    
}

void setPixel(byte x, byte y, byte type)
{
  switch(type)
  {
    case eOff: matrix->drawPixel(x,y,0); break;
    case eGreenLow: matrix->drawPixel(x,y,LED_GREEN_LOW); break;
    case eGreenHigh: matrix->drawPixel(x,y,LED_GREEN_HIGH); break;
  }

  //pixelArray[x][y] = type;
}

byte getPixel(byte x, byte y)
{
  long lColor;
  if(DebugIsActive)
  {
    Serial.print("x: ");
    Serial.print(x, DEC );
    Serial.print(" y: ");
    Serial.print(y, DEC );
  }
  
  if(y % 2 == 1) // each second row bc. of ZigZag LED arragement 
  {
    x = 10-x;
  }
  lColor = matrix->getPixelColor(y*11 + x);
  
  if(DebugIsActive)
  {
    Serial.print(" lColor: ");
    Serial.println(lColor, DEC );
  }
    
  if(lColor == 64768)
  {
    lColor = eGreenHigh;
  }

  if(lColor == 4096)
  {
    lColor = eGreenLow;
  }
  
  return lColor; // eOff;
}

void ShowMatrix()
{
  byte xCount = 0;
  int yCount = 0;

  ClockIsActive = false;
  Serial.println("tHe mAtRiX");
  matrix->clear();
  
  setPixel(0,1, eGreenHigh);
  setPixel(1,0, eGreenLow);
  setPixel(10,9, eGreenHigh);
  matrix->show();
  
  for(yCount = mh ; yCount >= 0; yCount--)
    for(xCount = 0 ; xCount < mw; xCount++)
      getPixel(xCount,yCount);
  //return;
        
  while(Serial.available() == 0)
  {
    //for(yCount = 0 ; yCount < mh; yCount++)
    for(yCount = mh ; yCount >= 0; yCount--)
    {
      for(xCount = 0 ; xCount < mw; xCount++)
      {
        // Check if there should be a line
        switch(getPixel(xCount, yCount))
        {
          case eOff: 
            if(yCount == 0)
            {
              if(random(0,100) < 20 )
              {
                setPixel(xCount, yCount, eGreenHigh);
              }
            }
            
              setPixel(xCount, yCount+1, eOff);
                        
            break;
  
          case eGreenLow: 
            if(yCount == 0)
            {
              if(random(0,100) < 20 )
              {
                setPixel(xCount, yCount, eOff);
              }
            }
                  
            break;
            
          case eGreenHigh: 
            if(yCount !=  mh)
            {
              setPixel(xCount, yCount, eGreenLow);
              setPixel(xCount, yCount+1, eGreenHigh);
            }
            else
            {
              setPixel(xCount, yCount, eGreenLow);
              setPixel(xCount, 0, eOff);
            }  
            break;
        }
      }
    }  
    
    matrix->show();
    delay(200);
  }
    
}

void ShowRainbow()
{
  byte xCount = 0;
  byte yCount = 0;
  ClockIsActive = false;
  
  Serial.println(F("The Rainbow"));
  matrix->clear();
         
  //while(true)
  //{
    for(yCount = 0 ; yCount < mh; yCount++)
    //for(yCount = mh ; yCount >= 0; yCount--)
    for(xCount = 0 ; xCount < mw; xCount++)
    {
      matrix->drawPixel(xCount,yCount,matrix->Color(23*(1+xCount), 23*(10-xCount), 24*(1+yCount)));
    }
matrix->show();
   // if(Serial.available() > 0)   
   //   break;
  //}
}

      
void showTime()
{
  int showingHours = myHours;
  
  matrix->clear();
  showWord(eEs);
  showWord(eIst);

  
  
  if(myMinutes < 3)
  {
    // Es ist X Uhr
    showWord(eUhr);
  }
  else
  {
    if(myMinutes < 7)
    {
      // Es ist 5 nach ...
      showWord(eFuenf);
      showWord(eNach);  
    }
    else
    {
      if(myMinutes < 13)
      {
        // Es ist Zehn nach
        showWord(eZehn);
        showWord(eNach);  
      }
      else
      {
        if(myMinutes < 18)
        {
          // es ist Viertel Stunde+1
          showWord(eViertel);
          showingHours++;
        }
        else
        {
          if(myMinutes < 23)
          {
            // Es ist Zwanzig nach
            showWord(eZwanzig);
            showWord(eNach);
          }
          else
          {
            if(myMinutes < 28)
            {
              // Es ist Zwanzig nach
              showWord(eFuenf);
              showWord(eVor);
              showWord(eHalb);
              showingHours++;
            }
            else
            {
              if(myMinutes < 33)
              {
                // Es ist halb ..
                showWord(eHalb);
                showingHours++;
              }
              else
              {
                if(myMinutes < 38)
                {
                  // Es ist fünf nach halb
                  showWord(eFuenf);
                  showWord(eNach); 
                  showWord(eHalb);
                  showingHours++;
                }
                else
                {
                  if(myMinutes < 38)
                  {
                    // Es ist fünf nach halb
                    showWord(eZehn);
                    showWord(eNach); 
                    showWord(eHalb);
                    showingHours++;
                  }
                  else
                  {
                    if(myMinutes < 43)
                    {
                      // Es ist zwanzig vor
                      showWord(eZwanzig);
                      showWord(eVor); 
                      showingHours++;
                    }
                    else
                    {
                      if(myMinutes < 43)
                      {
                        // Es ist Zehn nach halb
                        showWord(eZehn);
                        showWord(eNach); 
                        showWord(eHalb);
                        showingHours++;
                      }
                      else
                      {
                        if(myMinutes < 48)
                        {
                          // Es ist Dreiviertel ...
                          showWord(eDrei);
                          showWord(eViertel); 
                          showingHours++;
                        }
                        else
                        {
                          if(myMinutes < 53)
                          {
                            // Es ist Zehn vor
                            showWord(eZehn);
                            showWord(eVor); 
                            showingHours++;
                          }
                          else
                          {
                            if(myMinutes <= 60)
                            {
                              // Es ist Fünf vor
                              showWord(eFuenf);
                              showWord(eVor);
                              showingHours++;
                            }
                            else
                            {
                              // fuckfuckfuck
                            }
                          }
                        }
                      }
                  }
                }
              }
              }
            }
          }
        }
      }
    }
  }

  // Serial.println(showingHours);
  
  switch(showingHours)
  {
    case 0: showWord(eZwoelfUhr); break;
    case -1:
    case 1: if(myMinutes < 3) // Only full hour is "ein Uhr"
                showWord(eEinUhr); 
            else
                showWord(eEinsUhr); 
            break;
    case 2: showWord(eZweiUhr); break;
    case 3: showWord(eDreiUhr); break;
    case 4: showWord(eVierUhr); break;
    case 5: showWord(eFuenfUhr); break;
    case 6: showWord(eSechsUhr); break;
    case 7: showWord(eSiebenUhr); break;
    case 8: showWord(eAchtUhr); break;
    case 9: showWord(eNeunUhr); break;
    case 10: showWord(eZehnUhr); break; 
    case 11: showWord(eElfUhr); break;
    case 12: showWord(eZwoelfUhr); break;
  }
  
  matrix->show();
}

// Shows words on the Clock
void showWord(WordId myWordId)
{
  switch(myWordId)
  {
      case  eEs: matrix->fillRect(0,0, 2,0, myColor); break; // ES
      case  eIst: matrix->fillRect(3,0, 3,0, myColor); break; // Ist
      case  eFuenf: matrix->fillRect(7,0, 4,0, myColor); break; // Fünf
      case  eZehn: matrix->fillRect(0,1, 4,1, myColor); break; // Zehn
      case  eZwanzig: matrix->fillRect(4,1, 7,1, myColor); break; // Zwanzig 
      case  eDrei: matrix->fillRect(0,2, 4,1, myColor); break;   // Drei
      case  eViertel: matrix->fillRect(4,2, 7,1, myColor); break; // Viertel 
      case  eNach: matrix->fillRect(2,3, 4,1, myColor); break; // Nach
      case  eVor: matrix->fillRect(6,3, 3,1, myColor);  break; // Vor
      case  eJa: matrix->fillRect(9,3, 2,1, myColor);  break; // Ja
      case  eHalb: matrix->fillRect(0,4, 4,1, myColor); break; // Halb  
      case  eBL: matrix->fillRect(3,4, 2,1, myColor); break; // BL 
      case  eZwoelfUhr: matrix->fillRect(5,4, 5,1, myColor);  break; // Zwölf
      case  eZweiUhr: matrix->fillRect(0,5, 4,1, myColor); break; // Zwei 
      case  eEinUhr: matrix->fillRect(2,5, 3,1, myColor); break; // Zwei 
      case  eEinsUhr: matrix->fillRect(2,5, 4,1, myColor); break; // Zwei 
      case  eSiebenUhr: matrix->fillRect(5,5, 6,1, myColor); break; // Sieben  
      case  eDreiUhr: matrix->fillRect(1,6, 4,1, myColor);  break;  // Drei
      case  eJL: matrix->fillRect(5,6, 2,1, myColor); break; // JL
      case  eFuenfUhr: matrix->fillRect(7,6, 4,1, myColor); break; // Fünf 
      case  eElfUhr: matrix->fillRect(0,7, 3,1, myColor); break;  // Elf
      case  eNeunUhr: matrix->fillRect(3,7, 4,1, myColor); break; // Neun
      case  eVierUhr: matrix->fillRect(7,7, 4,1, myColor); break; // Vier
      case  eAchtUhr: matrix->fillRect(1,8, 4,1, myColor); break; // Acht 
      case  eZehnUhr: matrix->fillRect(5,8, 4,1, myColor); break; // Zehn 
      case  eLL: matrix->fillRect(9,8, 2,1, myColor); break; // LL 
      case  eSechsUhr: matrix->fillRect(1,9, 5,1, myColor); break; // Sechs
      case  eUhr: matrix->fillRect(8,9, 3,1, myColor);  break; // Uhr
  }  
}

void setColorBySerial()
{
  
  byte myTempR = 0;     // variable for temp data Red  
  byte myTempG = 0;     // variable for temp data Green  
  byte myTempB = 0;     // variable for temp data Blue  

  incomingByteCount = 0;
  while (Serial.available()<10)
  {
    // Wait...
  }
  myTemp = 0;
  while (Serial.available()>0)
  {
    // Wait...
    incomingByte = Serial.read();
    

    if(incomingByte >= '0')
    {
      incomingByteCount++;
      switch(incomingByteCount)
      {
        // Red
        case 1: myTemp = (incomingByte - '0') * 100; break;
        case 2: myTemp += (incomingByte - '0') * 10; break;
        case 3: myTemp += (incomingByte - '0'); 
                myTempR = myTemp & 0xFF;
                if(DebugIsActive)
                {
                  Serial.print(F(" Color R: "));
                  Serial.print(myTemp, HEX);
                  Serial.print(" : ");
                  Serial.println(myColor, HEX);
                }
                break;
        // Green
        case 4: myTemp = (incomingByte - '0') * 100; break;
        case 5: myTemp += (incomingByte - '0') * 10; break;
        case 6: myTemp += (incomingByte - '0'); 
                myTempG = myTemp & 0xFF;
                if(DebugIsActive)
                {
                  Serial.print(F(" Color G: "));
                  Serial.print(myTemp, HEX);
                  Serial.print(" : ");
                  Serial.println(myColor, HEX);
                }
                
                break;
       // Blue
        case 7: myTemp = (incomingByte - '0') * 100; break;
        case 8: myTemp += (incomingByte - '0') * 10; break;
        case 9: myTemp += (incomingByte - '0'); 
                myTempB = myTemp & 0xFF;

                myColor = matrix->Color(myTempR, myTempG, myTempB);
                if(DebugIsActive)
                {
                  Serial.print(F(" Color B: "));
                  Serial.print(myTemp, HEX);
                }
      }
    }
  }
  if(DebugIsActive)
  {
    Serial.print(F(" Done: "));
    Serial.println(myColor, HEX);
  }
}

void loop() {
  
  // reply only when you receive data:
  if (Serial.available() > 0) {
    // read the incoming byte:
    incomingByte = Serial.read(); 
    
    
    // evaluate what you got    
    switch(incomingByte)
    {
      
      case 'i': 
      case '?': sendInfoBySerial();
                break;

      case 'd': DebugIsActive = !DebugIsActive;
                if(DebugIsActive)
                  Serial.println("Debug Mode Active");
                else
                  Serial.println("Debug Mode Inactive");
                break;

      case 'k': Serial.println("Displaying Circles");
                //display_circles();
                break;
                
      case 'r': sendActualTimeBySerial(); break;

      case 'u': if(DebugIsActive)
                  Serial.println(F("Showing Time on Display"));
                showTime(); break;
      
      case 'f': matrix->fillScreen(myColor);
                matrix->show();
                break;
                
      case 'x': matrix->clear();
                matrix->show();
                Serial.println("Cleared Clock Face");
                break;
                
      case 'b': Serial.println("Set Brightness (000 - 255)");
                incomingByteCount = 0;
                CommandComplete = false;
                
                while (Serial.available()<4)
                {
                  // Wait...
                }
                while (Serial.available()>0 && not CommandComplete)
                {
                  // Wait...
                  incomingByte = Serial.read();
                  

                  if(incomingByte >= '0')
                  {
                    incomingByteCount++;
                    switch(incomingByteCount)
                    {
                      case 1: xPos = (incomingByte - '0') * 100; break;
                      case 2: xPos += (incomingByte - '0') * 10; break;
                      case 3: xPos += (incomingByte - '0') * 1; break;
                              if(xPos > 255)
                              {
                                xPos = xPos % 256;
                                Serial.println("Invalid Brightness!");
                              }
                              CommandComplete = true;
                              break;
                              
                    }
                  }

                  Serial.print(incomingByteCount, DEC);
                  Serial.print("  ");
                  Serial.print(incomingByte, DEC);
                  Serial.print("  ");
                  Serial.println(incomingByte - '0', DEC);
                }
                Serial.print("Brightness:  ");
                Serial.println(xPos, DEC);
                matrix->setBrightness(xPos);
                
                break;          
      case 'h': ClockIsActive = !ClockIsActive;
                if(ClockIsActive)
                {
                  Serial.println("Clock updating active");
                }
                else
                {
                  Serial.println("Clock updating halted");
                }
                break;

      case 'p': // Set up individual pixel
                //Serial.println("Please enter coordinates xxyy :"); // 11:11:11
                incomingByteCount = 0;
                CommandComplete = false;
                
                while (Serial.available()<4)
                {
                  // Wait...
                }
                while (Serial.available()>0 && not CommandComplete)
                {
                  // Wait...
                  incomingByte = Serial.read();
                  

                  if(incomingByte >= '0')
                  {
                    incomingByteCount++;
                    switch(incomingByteCount)
                    {
                      case 1: xPos = (incomingByte - '0') * 10; break;
                      case 2: xPos = (incomingByte - '0') + xPos;
                              if(xPos > 10)
                              {
                                xPos = xPos % 10;
                                Serial.println("Invalid xPos!");
                              }
                              
                              break;
                              
                      
                      case 3: yPos = (incomingByte - '0') * 10; break;
                      case 4: yPos = (incomingByte - '0') + yPos;
                              if(yPos > 9)
                              {
                                yPos = yPos % 9;
                                Serial.println("Invalid yPos!");
                              }
                              CommandComplete = true;
                              break;
                              
                    }
                  }
                  //Serial.print(incomingByteCount, DEC);
                  //Serial.print("  ");
                  //Serial.print(incomingByte, DEC);
                  //Serial.print("  ");
                  //Serial.println(incomingByte - '0', DEC);
                  
                  
                }
                //Serial.print("xPos:  ");
                //Serial.print(xPos, DEC);
                //Serial.print(" yPos: ");
                //Serial.println(yPos, DEC);
                matrix->fillRect(xPos,yPos, 1,1, myColor);
                matrix->show();
                break;
                
      case 't': // Set up the Time by Serial Command
                Serial.println("Please enter actual Time hh:mm:ss :"); // 11:11:11
                incomingByteCount = 0;
                while (Serial.available()<8)
                {
                  // Wait...
                }
                while (Serial.available()>0)
                {
                  // Wait...
                  incomingByte = Serial.read();
                  

                  if(incomingByte >= '0')
                  {
                    incomingByteCount++;
                    switch(incomingByteCount)
                    {
                      case 1: myHours = (incomingByte - '0') * 10; break;
                      case 2: myHours = (incomingByte - '0') + myHours;
                              if(myHours >= 12)
                              {
                                myHours = myHours - 12; break;
                              }
                      case 3: break; // :
                      case 4: myMinutes = (incomingByte - '0') * 10; break;
                      case 5: myMinutes = (incomingByte - '0') + myMinutes; break;
                      case 6: break; // :
                      case 7: mySeconds = (incomingByte - '0') * 10; break;
                      case 8: mySeconds = (incomingByte - '0') + mySeconds; break;
                    }
                  }
                  //Serial.print(incomingByteCount, DEC);
                  //Serial.print("  ");
                  //Serial.print(incomingByte, DEC);
                  //Serial.print("  ");
                  //Serial.println(incomingByte - '0', DEC);
                }
                Serial.print(" Done: ");
                sendActualTimeBySerial();
                showTime();
                break;

      case 'c': // Set up the Color by Serial
                //myColor = (255 << 11) + (100 << 5) + 12;
                
                if(DebugIsActive) Serial.println("Please enter Color RRRGGGBBB :"); // 255255255
                
                setColorBySerial();
                
                break;

     case 'l': ShowEachLine(1000);
               break;

     case 'm': 
                // Get rid of unneccessary characters
                while(Serial.available() > 0) 
                  Serial.read();
                ShowMatrix();
               break;

     case 'n': 
                // Get rid of unneccessary characters
                while(Serial.available() > 0) 
                  Serial.read();
                ShowRainbow();
               break;
    }
  }

  
  delay(100);

  // If the Clock is Active...
  if(ClockIsActive)
  {
    // Check if another Minute has passed... then recalc.
    if(myMinutesLastDisplayed != myMinutes)
    {
      myMinutesLastDisplayed = myMinutes; 
      // Show the Time
      showTime();
    }  
  }

}

void setup() {
    Serial.begin(9600);
    matrix->begin();
    matrix->setTextWrap(false);
    matrix->setBrightness(BRIGHTNESS);
    // Test full bright of all LEDs. If brightness is too high
    // for your current limit (i.e. USB), decrease it.
    matrix->clear();

    matrix->fillScreen(LED_GREEN_LOW);
    matrix->show();
    delay(3000);
    
    // Show the Time
    showTime();

    Timer1.initialize(1*1000000); // Every 1 second
    //Timer1.initialize(1*1000); // Every 1 second
    Timer1.attachInterrupt(countSecond);
    ClockIsActive = true;

    //randomSeed(analogRead(0));

    //DebugIsActive = true;
    //ShowMatrix(1000);
}

// This Timer is executed every second
void countSecond()
{
  mySeconds++;
  if(mySeconds >= 60)
  {
    mySeconds = 0; // mySeconds % 60;
    myMinutes++;
       
    if(myMinutes >= 60)
    {
      myMinutes = 0;
      myHours++;
      if(myHours >= 12)
      {
        myHours = 0;
        myHalfDays++;
      }
    }
  }  
}
